<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

//	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");	
		
	/*
	 * 
	 */
	class PolarisLinkStockManager
	{
        /*
            Obtient la liste des boutiques mappées
        */
        public static function GetMappedStore($langId = NULL, $allStore = false)
        {
            $stores = [];

            if ($langId === NULL)
                $langId = Configuration::get("PS_LANG_DEFAULT");

            $storesPresta = [];

            if (version_compare(_PS_VERSION_, 1.7) < 0)
                $query = "SELECT * FROM "._DB_PREFIX_."store AS s ORDER BY id_store ASC";
            else
                $query = "SELECT * FROM "._DB_PREFIX_."store AS s LEFT JOIN "._DB_PREFIX_."store_lang AS sl ON (sl.id_store = s.id_store AND sl.id_lang = ".$langId.") ORDER BY s.id_store ASC";

            foreach (Db::getInstance()->executeS($query) as $store)
                $storesPresta["ps".$store["id_store"]] = $store;

            $mapCfg = Configuration::get("POLARISLINK_STORESMAP");
            if (!empty($mapCfg))
            {
                $storesMapCfg = json_decode($mapCfg, true);
                if (!empty($storesMapCfg))
                {
                    foreach ($storesMapCfg as $refPolaris => $refPresta)
                        if (isset($storesPresta["ps".$refPresta]))
                            $storesPresta["ps".$refPresta]["code_polaris"] = substr($refPolaris, 5);
                }
            }

            // Filtrage des magasins non polaris
            foreach ($storesPresta as &$store)
            {
                if ($allStore || isset($store["code_polaris"]))
                    $stores[] = $store;
            }

            return $stores;
        }            

        /*
            Calcule et renvoi les informations de dispatch commandes pour une commande
            précise
        */
        public static function OrderGetDispatchInfo($order, $langId = NULL)
        {
            $db = Db::getInstance();

            $id_order = $order->id;
            
            // Chargement des boutiques mappées
            $stores = self::GetMappedStore($langId);

            // Chargement du magasin de commande
            $orderStoreId = NULL;
            /*foreach ($db->executeS("SELECT id_store FROM "._DB_PREFIX_."orders WHERE id_order = ".$id_order) as $row)
                $orderStoreId = (int) $row["id_store"];*/

            // Préparation des scores
            foreach ($stores as &$store)
            {
                $store["score"] = 0;
                $store["a_qty"] = null;
            }

            // Chargement du détail de la commande
            $orderDetails = [];
            $id_products = [];
            foreach (OrderDetail::getList($id_order) as &$detail)
            {
                $id_products[] = (int) $detail["product_id"];
                $orderDetails[] = [
                    "product_id" => (int) $detail["product_id"],
                    "product_attribute_id" => (int) $detail["product_attribute_id"],
                    "qty" => (int) $detail["product_quantity"],
                    "name" => $detail["product_reference"] . " - " . $detail["product_name"],
                    "ean" => $detail["product_supplier_reference"],
                    "stocks" => [],
                ];
            }

            // Interrogation stock
            $stocks = self::GetMultiStock($id_products);

            // On ajoute les stocks de la commande aux stocks décrémentés
            if (!empty($orderStoreId))
            {
                // Mais seulement aux commandes dont l'état le nécessaite
                $status = new OrderState($order->current_state);
                if (!$status->shipped)
                {
                    foreach ($orderDetails as &$detail)
                        $stocks[$detail["product_id"]][$detail["product_attribute_id"]][$orderStoreId] += $detail["qty"];
                }
            }
            
            // Distribution des stocks par magasin, pour chaque détail de la commande
            foreach ($orderDetails as &$detail)
            {
                foreach ($stores as &$store)
                {                    
                    $qty = 0;
                    if (isset($stocks[$detail["product_id"]][$detail["product_attribute_id"]][$store["id_store"]]))
                        $qty = (int) $stocks[$detail["product_id"]][$detail["product_attribute_id"]][$store["id_store"]];

                    $detail["stocks"][$store["id_store"]] = $qty;

                    if ($qty >= $detail["qty"])
                    {
                        // 10000 points si en stock
                        $store["score"] += 10000;

                        // 1 point supplémentaire si on a une quantité supplémentaire de sécurité
                        if ($qty > $detail["qty"])
                            $store["score"] += 1;
                    }

                    $nb = 1;
                    if ($store["a_qty"] !== NULL)
                        $nb++;
                    
                    $store["a_qty"] = ($store["a_qty"] + $qty) / (double) $nb;
                }
            }

            // Tri des magasins par score
            uasort($stores, function ($a, $b) {
                if ($a["score"] > $b["score"])
                    return -1;
                if ($a["score"] < $b["score"])
                    return 1;
                
                if ($a["a_qty"] > $b["a_qty"])
                    return -1;
                if ($a["a_qty"] < $b["a_qty"])
                    return 1;
                
                return 0;    
            });

            return [ 
                "current_store" => $orderStoreId,
                "stores" => $stores, 
                "details" => $orderDetails 
            ];
        }

        /*
            Obtient le relevé du stock pour une ou plusieurs références 
            $id_product est soit un tableau, soit une donnée

            retour :
            [
                id_product => 
                [
                    id_product_attribute / 0 => [
                        id_store => stock
                    ]
                ]
            ]
        */
        public static function GetMultiStock($id_product)
        {            
            $stocks = [];
            if (!empty($id_product))
            {
                $query = "SELECT id_product, COALESCE(id_product_attribute, 0) AS id_product_attribute, id_store, stock FROM "._DB_PREFIX_."polarislink_stocks WHERE id_product";
                if (is_array($id_product))
                    $query .= " IN (".implode(",", $id_product).")";
                else
                    $query .= " = ".$id_product;

                $db = Db::getInstance();
                if ($results = $db->executeS($query))
                {
                    foreach ($results as $row)
                    {
                        if (!isset($results[$row["id_product"]]))
                            $results[$row["id_product"]] = [];
                        
                        if (!isset($results[$row["id_product"]][$row["id_product_attribute"]]))
                            $results[$row["id_product"]][$row["id_product_attribute"]] = [];
                            
                        $stocks[$row["id_product"]][$row["id_product_attribute"]][$row["id_store"]] = (int) $row["stock"];
                    }
                }
            }

            return $stocks;
        }

		public static function SetMultiStock($id_store, $id_product, $id_product_attribute, $qty, $relative = false)
		{
			$db = Db::getInstance();

            if (!$db->update('polarislink_stocks', 
					array(
						'stock' => $relative ? array('type' => 'sql', 'value' => 'stock + ' . (int) $qty) : (int) $qty,
						'date_upd' => array('type' => 'sql', 'value' => "NOW()"),
					), 
					sprintf("id_product = %d AND id_product_attribute %s AND id_store = %d",
					$id_product,
					empty($id_product_attribute) ? " IS NULL" : " = ".$id_product_attribute,
					$id_store
			) ))
			{
				// Problème base, pas le bon schéma, on suspend tout !!
				throw new PrestaShopDatabaseException($db->getMsgError());
			}

			if ($db->numRows() == 0)
			{
				// Aucune donnée n'a été modifiée car pas trouvé (assuré par le date_upd toujours différent)
				// donc on doit créer la référence
				$nb = $db->insert('polarislink_stocks', 
						array(
							'id_product' => $id_product,
							'id_product_attribute' => empty($id_product_attribute) ? array('type' => 'sql', 'value' => "NULL") : $id_product_attribute,
							'id_store' => $id_store,
							'stock' => (int) $qty,
							'date_upd' => array('type' => 'sql', 'value' => "NOW()"),
                        ));
                echo $db->getMsgError();
			}
		}		
	}
	